@@ -43,6 +43,7 @@
 
 #include <gdbus.h>
 
+#include "hcid.h"
 #include "dbus-common.h"
 #include "logging.h"
 #include "adapter.h"
@@ -60,38 +61,21 @@ const char *manager_get_base_path(void)
 	return base_path;
 }
 
-int manager_update_adapter(uint16_t dev_id, uint8_t svc, gboolean starting)
-{
-	struct btd_adapter *adapter;
-
-	adapter = manager_find_adapter_by_id(dev_id);
-	if (!adapter)
-		return -EINVAL;
-
-	return adapter_update(adapter, svc, starting);
-}
-
-int manager_startup_complete(void)
+void manager_update_svc(const bdaddr_t *bdaddr, uint8_t svc)
 {
 	GSList *l;
+	bdaddr_t src;
 
 	for (l = adapters; l != NULL; l = l->next) {
 		struct btd_adapter *adapter = l->data;
-		adapter_update(adapter, 0, FALSE);
+
+		adapter_get_address(adapter, &src);
+
+		if (bacmp(bdaddr, BDADDR_ANY) != 0 && bacmp(bdaddr, &src) != 0)
+			continue;
+
+		adapter_update(adapter, svc);
 	}
-
-	return 0;
-}
-
-int manager_get_adapter_class(uint16_t dev_id, uint8_t *cls)
-{
-	struct btd_adapter *adapter;
-
-	adapter = manager_find_adapter_by_id(dev_id);
-	if (!adapter)
-		return -EINVAL;
-
-	return adapter_get_class(adapter, cls);
 }
 
 static inline DBusMessage *invalid_args(DBusMessage *msg)
@@ -227,6 +211,10 @@ static DBusMessage *get_properties(DBusConnection *conn,
 	array = g_new0(char *, g_slist_length(adapters) + 1);
 	for (i = 0, list = adapters; list; list = list->next, i++) {
 		struct btd_adapter *adapter = list->data;
+
+		if (!adapter_is_ready(adapter))
+			continue;
+
 		array[i] = (char *) adapter_get_path(adapter);
 	}
 	dict_append_array(&dict, "Adapters", DBUS_TYPE_OBJECT_PATH, &array, i);
@@ -273,6 +261,10 @@ static void manager_update_adapters(void)
 	array = g_new0(char *, g_slist_length(adapters) + 1);
 	for (i = 0, list = adapters; list; list = list->next, i++) {
 		struct btd_adapter *adapter = list->data;
+
+		if (!adapter_is_ready(adapter))
+			continue;
+
 		array[i] = (char *) adapter_get_path(adapter);
 	}
 
@@ -304,6 +296,9 @@ static void manager_remove_adapter(struct btd_adapter *adapter)
 			DBUS_TYPE_INVALID);
 
 	adapter_remove(adapter);
+
+	if (adapters == NULL)
+		btd_start_exit_timer();
 }
 
 void manager_cleanup(DBusConnection *conn, const char *path)
@@ -405,24 +400,16 @@ GSList *manager_get_adapters(void)
 	return adapters;
 }
 
-static void manager_add_adapter(struct btd_adapter *adapter)
+void manager_add_adapter(const char *path)
 {
-	const gchar *path = adapter_get_path(adapter);
-
-	adapters = g_slist_append(adapters, adapter);
-
 	g_dbus_emit_signal(connection, "/",
 			MANAGER_INTERFACE, "AdapterAdded",
 			DBUS_TYPE_OBJECT_PATH, &path,
 			DBUS_TYPE_INVALID);
 
-	if (default_adapter_id < 0) {
-		int new_default = hci_get_route(NULL);
-
-		manager_set_default_adapter(new_default);
-	}
-
 	manager_update_adapters();
+
+	btd_stop_exit_timer();
 }
 
 int manager_register_adapter(int id, gboolean devup)
@@ -439,7 +426,7 @@ int manager_register_adapter(int id, gboolean devup)
 	if (!adapter)
 		return -1;
 
-	manager_add_adapter(adapter);
+	adapters = g_slist_append(adapters, adapter);
 
 	return 0;
 }
@@ -464,7 +451,7 @@ int manager_unregister_adapter(int id)
 
 int manager_start_adapter(int id)
 {
-	struct btd_adapter* adapter;
+	struct btd_adapter *adapter;
 	int ret;
 
 	adapter = manager_find_adapter_by_id(id);
@@ -520,3 +507,17 @@ void manager_set_default_adapter(int id)
 			DBUS_TYPE_OBJECT_PATH, &path,
 			DBUS_TYPE_INVALID);
 }
+
+void btd_manager_set_offline(gboolean offline)
+{
+	GSList *l;
+
+	for (l = adapters; l != NULL; l = g_slist_next(l)) {
+		struct btd_adapter *adapter = l->data;
+
+		if (offline)
+			btd_adapter_switch_offline(adapter);
+		else
+			btd_adapter_restore_powered(adapter);
+	}
+}
